@@ -33,7 +33,7 @@
  */
 
 #include "env.h"
-#include <p8-platform/util/StringUtils.h>
+#include "p8-platform/util/StringUtils.h"
 
 namespace CEC
 {
@@ -185,6 +185,8 @@ namespace CEC
         return "1.3a";
       case CEC_VERSION_1_4:
         return "1.4";
+      case CEC_VERSION_2_0:
+        return "2.0";
       default:
         return "unknown";
       }
@@ -497,6 +499,8 @@ namespace CEC
       case CEC_VENDOR_TOSHIBA:
       case CEC_VENDOR_TOSHIBA2:
         return "Toshiba";
+      case CEC_VENDOR_APPLE:
+        return "Apple";
       case CEC_VENDOR_AKAI:
         return "Akai";
       case CEC_VENDOR_AOC:
@@ -766,6 +770,10 @@ namespace CEC
         return "Raspberry Pi";
       case ADAPTERTYPE_TDA995x:
         return "TDA995x";
+      case ADAPTERTYPE_IMX:
+        return "i.MX";
+      case ADAPTERTYPE_LINUX:
+        return "Linux";
       default:
         return "unknown";
       }
