@@ -79,6 +79,23 @@ void libcec_clear_configuration(libcec_configuration* configuration)
     configuration->Clear();
 }
 
+#if CEC_LIB_VERSION_MAJOR >= 5
+int libcec_set_callbacks(libcec_connection_t connection, ICECCallbacks* callbacks, void* cbParam)
+{
+  ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
+  if (adapter)
+    return adapter->SetCallbacks(callbacks, cbParam) ? 1 : 0;
+  return -1;
+}
+
+int libcec_disable_callbacks(libcec_connection_t connection)
+{
+  ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
+  if (adapter)
+    return adapter->DisableCallbacks() ? 1 : 0;
+  return -1;
+}
+#else
 int libcec_enable_callbacks(libcec_connection_t connection, void* cbParam, ICECCallbacks* callbacks)
 {
   ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
@@ -86,6 +103,7 @@ int libcec_enable_callbacks(libcec_connection_t connection, void* cbParam, ICECC
     return adapter->EnableCallbacks(cbParam, callbacks) ? 1 : 0;
   return -1;
 }
+#endif
 
 int8_t libcec_find_adapters(libcec_connection_t connection, cec_adapter* deviceList, uint8_t iBufSize, const char* strDevicePath)
 {
@@ -324,6 +342,7 @@ int libcec_volume_down(libcec_connection_t connection, int bSendRelease)
       -1;
 }
 
+#if CEC_LIB_VERSION_MAJOR >= 5
 int libcec_mute_audio(libcec_connection_t connection, int UNUSED(bSendRelease))
 {
   ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
@@ -331,6 +350,7 @@ int libcec_mute_audio(libcec_connection_t connection, int UNUSED(bSendRelease))
       adapter->AudioToggleMute() :
       -1;
 }
+#endif
 
 int libcec_send_keypress(libcec_connection_t connection, cec_logical_address iDestination, cec_user_control_code key, int bWait)
 {
@@ -354,7 +374,10 @@ int libcec_get_device_osd_name(libcec_connection_t connection, cec_logical_addre
   if (!!adapter)
   {
     std::string osdName(adapter->GetDeviceOSDName(iAddress));
-    strncpy(name, osdName.c_str(), std::min(sizeof(cec_osd_name), osdName.size()));
+    size_t osd_size(osdName.size());
+    memcpy(name, osdName.c_str(), std::min(sizeof(cec_osd_name), osd_size));
+    if (osd_size < sizeof(cec_osd_name))
+      name[osd_size] = (char)0;
     return 0;
   }
   return -1;
@@ -394,6 +417,15 @@ int libcec_get_current_configuration(libcec_connection_t connection, libcec_conf
       -1;
 }
 
+#if CEC_LIB_VERSION_MAJOR >= 5
+int libcec_can_save_configuration(libcec_connection_t connection)
+{
+  ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
+  return adapter ?
+    (adapter->CanSaveConfiguration() ? 1 : 0) :
+    -1;
+}
+#else
 int libcec_can_persist_configuration(libcec_connection_t connection)
 {
   ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
@@ -401,7 +433,9 @@ int libcec_can_persist_configuration(libcec_connection_t connection)
       (adapter->CanPersistConfiguration() ? 1 : 0) :
       -1;
 }
+#endif
 
+#if CEC_LIB_VERSION_MAJOR < 5
 int libcec_persist_configuration(libcec_connection_t connection, libcec_configuration* configuration)
 {
   ICECAdapter* adapter = static_cast<ICECAdapter*>(connection);
@@ -409,6 +443,7 @@ int libcec_persist_configuration(libcec_connection_t connection, libcec_configur
       (adapter->PersistConfiguration(configuration) ? 1 : 0) :
       -1;
 }
+#endif
 
 int libcec_set_configuration(libcec_connection_t connection, libcec_configuration* configuration)
 {
