@@ -164,7 +164,8 @@ bool CCECAdapterMessageQueueEntry::IsResponse(const CCECAdapterMessage &msg)
       thisMsgCode == MSGCODE_SET_OSD_NAME ||
       thisMsgCode == MSGCODE_WRITE_EEPROM ||
       thisMsgCode == MSGCODE_TRANSMIT_IDLETIME ||
-      thisMsgCode == MSGCODE_SET_ACTIVE_SOURCE)
+      thisMsgCode == MSGCODE_SET_ACTIVE_SOURCE ||
+      thisMsgCode == MSGCODE_SET_AUTO_POWER_ON)
     return thisMsgCode == msgResponse;
 
   if (!m_message->IsTransmission())
@@ -213,7 +214,8 @@ bool CCECAdapterMessageQueueEntry::MessageReceivedCommandAccepted(const CCECAdap
       if (!m_message->IsTransmission() && m_iPacketsLeft == 0)
       {
         m_message->state = ADAPTER_MESSAGE_STATE_SENT_ACKED;
-        m_message->response = message.packet;
+        memcpy(m_message->m_rx_data, message.m_tx_data, message.m_tx_len);
+        m_message->m_rx_len = message.m_tx_len;
         bSendSignal = true;
       }
       bHandled = true;
@@ -237,7 +239,9 @@ bool CCECAdapterMessageQueueEntry::MessageReceivedTransmitSucceeded(const CCECAd
       m_queue->m_com->m_callback->GetLib()->AddLog(CEC_LOG_DEBUG, "%s - transmit succeeded", m_message->ToString().c_str());
 #endif
       m_message->state = ADAPTER_MESSAGE_STATE_SENT_ACKED;
-      m_message->response = message.packet;
+      memcpy(m_message->m_rx_data, message.m_tx_data, message.m_tx_len);
+      m_message->m_rx_len = message.m_tx_len;
+      m_queue->m_com->OnTxAck();
     }
     else
     {
@@ -261,13 +265,31 @@ bool CCECAdapterMessageQueueEntry::MessageReceivedResponse(const CCECAdapterMess
     m_queue->m_com->m_callback->GetLib()->AddLog(CEC_LOG_DEBUG, "%s - received response - %s", ToString(), message.ToString().c_str());
 #else
     if (message.IsError())
+    {
       m_queue->m_com->m_callback->GetLib()->AddLog(CEC_LOG_DEBUG, "%s - received response - %s", ToString(), message.ToString().c_str());
+      if (m_message->IsTransmission() && (message.Message() != MSGCODE_TRANSMIT_FAILED_ACK))
+        m_queue->m_com->OnTxError();
+    }
 #endif
-    m_message->response = message.packet;
+    memcpy(m_message->m_rx_data, message.m_tx_data, message.m_tx_len);
+    m_message->m_rx_len = message.m_tx_len;
     if (m_message->IsTransmission())
-      m_message->state = message.Message() == MSGCODE_TRANSMIT_SUCCEEDED ? ADAPTER_MESSAGE_STATE_SENT_ACKED : ADAPTER_MESSAGE_STATE_SENT_NOT_ACKED;
+    {
+      if (message.Message() == MSGCODE_TRANSMIT_SUCCEEDED)
+      {
+        m_message->state = ADAPTER_MESSAGE_STATE_SENT_ACKED;
+        m_queue->m_com->OnTxAck();
+      }
+      else
+      {
+        m_message->state = ADAPTER_MESSAGE_STATE_SENT_NOT_ACKED;
+        m_queue->m_com->OnTxNack();
+      }
+    }
     else
+    {
       m_message->state = ADAPTER_MESSAGE_STATE_SENT_ACKED;
+    }
   }
 
   Signal();
@@ -365,7 +387,7 @@ void CCECAdapterMessageQueue::MessageReceived(const CCECAdapterMessage &msg)
   bool bHandled(false);
   CLockObject lock(m_mutex);
   /* send the received message to each entry in the queue until it is handled */
-  for (std::map<uint64_t, CCECAdapterMessageQueueEntry *>::iterator it = m_messages.begin(); !bHandled && it != m_messages.end(); it++)
+  for (auto it = m_messages.begin(); !bHandled && it != m_messages.end(); ++it)
     bHandled = it->second->MessageReceived(msg);
 
   if (!bHandled)
@@ -376,7 +398,10 @@ void CCECAdapterMessageQueue::MessageReceived(const CCECAdapterMessage &msg)
     m_com->m_callback->GetLib()->AddLog(bIsError ? CEC_LOG_WARNING : CEC_LOG_DEBUG, msg.ToString().c_str());
 #else
     if (bIsError)
+    {
+      m_com->OnRxError();
       m_com->m_callback->GetLib()->AddLog(CEC_LOG_WARNING, msg.ToString().c_str());
+    }
 #endif
 
     /* push this message to the current frame */
@@ -384,7 +409,10 @@ void CCECAdapterMessageQueue::MessageReceived(const CCECAdapterMessage &msg)
     {
       /* and push the current frame back over the callback method when a full command was received */
       if (m_com->IsInitialised())
+      {
+        m_com->OnRxSuccess();
         m_com->m_callback->OnCommandReceived(m_currentCECFrame);
+      }
 
       /* clear the current frame */
       m_currentCECFrame.Clear();
@@ -392,25 +420,14 @@ void CCECAdapterMessageQueue::MessageReceived(const CCECAdapterMessage &msg)
   }
 }
 
-void CCECAdapterMessageQueue::AddData(uint8_t *data, size_t iLen)
+void CCECAdapterMessageQueue::AddData(uint8_t *data, size_t len)
 {
-  for (size_t iPtr = 0; iPtr < iLen; iPtr++)
+  for (size_t ptr = 0; ptr < len; ++ptr)
   {
-    bool bFullMessage(false);
+    if (m_incomingAdapterMessage->PushReceivedByte(data[ptr]))
     {
-      CLockObject lock(m_mutex);
-      bFullMessage = m_incomingAdapterMessage->PushReceivedByte(data[iPtr]);
-    }
-
-    if (bFullMessage)
-    {
-      /* a full message was received */
-      CCECAdapterMessage newMessage;
-      newMessage.packet = m_incomingAdapterMessage->packet;
-      MessageReceived(newMessage);
-
-      /* clear the current message */
-      CLockObject lock(m_mutex);
+      // a full message was received
+      MessageReceived(*m_incomingAdapterMessage);
       m_incomingAdapterMessage->Clear();
     }
   }
